#include "SoXtWebViewer.h"
#include "WebCB.h"
#include <Inventor/Xt/SoXt.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoWWWInline.h>
#include <Inventor/nodes/SoCamera.h>
#include <Inventor/fields/SoSFRotation.h>
#include <Inventor/SbLinear.h>
#include <Xm/PushBG.h>
#include <Xm/MessageB.h>
#include <iostream.h>

#undef DEBUG


SoXtWebViewer::SoXtWebViewer( int status,
			      int md,
			      Widget parent, 
			      const char *name, 
			      SbBool buildInsideParent, 
			      SoXtFullViewer::BuildFlag flag, 
			      SoXtViewer::Type type )
  : SoXtWalkViewer( parent, name, buildInsideParent, flag, type, TRUE )
{ 
  OLDlg = new SoXtOLDialog( this, parent );
  //  actualPos = getCamera()->position.getValue();
  //  actualOrt = getCamera()->orientation.getValue();
  master = status;
  update = 0;
  mode = md;
  if( ! master )
    setDecoration( FALSE );
  
  //GIS protocol handler initialisation
  if( fcntl( SOCK, F_SETFL, FNDELAY ) < 0 )
    {
      perror( "F_SETFL:FNDELAY" );
      exit(1);
    }
  XtAppAddInput( SoXt::getAppContext(), SOCK,
		 ( XtPointer )XtInputReadMask, sigio_handler, NULL );
}


void
SoXtWebViewer::redraw()
{
  static char header = CAMP;
  static float cPrms[7];
  
  if( master )
    {
      getCamera()->position.getValue().getValue( cPrms[0], cPrms[1], cPrms[2] );
      getCamera()->orientation.getValue().getValue( cPrms[3], cPrms[4], cPrms[5], cPrms[6] );
      write( SOCK, &header, 1 );
      write( SOCK, cPrms, sizeof( float[7] ) );
#ifdef DEBUG
      printf( "\nMaster %d: CAMP:\t(%f, %f, %f)\n\t\t\t(%f, %f, %f, %f)\n", getpid(),
	      cPrms[0], cPrms[1], cPrms[2], cPrms[3], cPrms[4], cPrms[5], cPrms[6] );
      fflush( stdout );
#endif
    }

#ifdef DEBUG
  fprintf( stderr, "proc: %d, from sigio_handler: %d\n", getpid(), update );
#endif

  SoXtWalkViewer::redraw();

#ifdef DEBUG
  fprintf( stderr, "proc: %d, after redraw()\n", getpid() );
#endif
}
    
void 
SoXtWebViewer::sigio_handler( XtPointer, int*, XtInputId* )
{
  static char header;
  static char len;
  static CamParams cPrms;
  static char newURL[URL_MAXLEN];

#ifdef DEBUG
  pid_t myPid = getpid();
#endif

  header = NHEAD;
	  
  for( ; ; )
    {
      if( read( SOCK, &header, 1 ) < 0 )
	{
#ifdef DEBUG
	  fprintf( stderr, "Proc %d: no more msg\n", myPid );
#endif
	  break;
	}
      switch( header )
	{
	case CAMP: 
	  read( SOCK, cPrms, sizeof( cPrms ) );
#ifdef DEBUG
	  fprintf( stderr, "proc: %d, CAMP\n", myPid );
#endif
	  break;

	case NOBJ:
#ifdef DEBUG
	  fprintf( stderr, "proc: %d, NOBJ\n", myPid );
#endif
	  read( SOCK, &len, 1 );
	  read( SOCK, newURL, len );
#ifdef DEBUG
	  fprintf( stderr, "proc: %d, NURL, len %d\n", myPid, len );
#endif
	  AddMovingObject( newURL, Viewer );
	  break;
	  
	case CFLY:
	  read( SOCK, cPrms, sizeof( cPrms ) );
	  Viewer->mode = FLY;
#ifdef DEBUG
	  fprintf( stderr, "proc: %d, CFLY\n", myPid );
#endif
	  break;

	case CWALK:
	  read( SOCK, cPrms, sizeof( cPrms ) );
	  Viewer->mode = WALK;
#ifdef DEBUG
	  fprintf( stderr, "proc: %d, CWALK\n", myPid );
#endif
	  break;

	case NURL:
	  read( SOCK, &len, 1 );
	  read( SOCK, newURL, len );
#ifdef DEBUG
	  fprintf( stderr, "proc: %d, NURL, len %d\n", myPid, len );
#endif
	  LoadGraph( newURL, Viewer );
	  break;
	default:
	  fprintf( stderr, "proc: %d, GIS protocol violation: %d\t", getpid(), header );
	  fprintf( stderr, "pruning of next %d bytes\n", read( SOCK, newURL, URL_MAXLEN ) );
	}
    }

  switch( header )
    {
    case NHEAD: 
      return;
      
    case CAMP: 
      Viewer->getCamera()->position.setValue( cPrms ); 
      Viewer->getCamera()->orientation.setValue( &cPrms[3] ); 
      if( Viewer->mode == FLY )
	Viewer->tiltCamera( -180 );
#ifdef DEBUG
      SbVec3f dir;
      float ang;
      fprintf( stderr, "Proc %d: pos:\t(%f, %f, %f)\n", myPid, cPrms[0], cPrms[1], cPrms[2] );
      Viewer->getCamera()->orientation.getValue( dir, ang );
      static const float* vec = dir.getValue();
      fprintf( stderr, "\t\t\t(%f, %f, %f, %f)\n", vec[0], vec[1], vec[2], ang );
#endif
      break;

#ifdef DEBUG
    case NURL:
      fprintf( stderr, "proc %d: New URL: %s\n", myPid, newURL );
      break;
#endif
      
    case CFLY:
#ifdef DEBUG
      fprintf( stderr, "Proc %d: CFLY received\n", myPid );
#endif
      Viewer->getCamera()->position.setValue( cPrms ); 
      Viewer->getCamera()->orientation.setValue( &cPrms[3] ); 
      Viewer->tiltCamera( -180 );
      Viewer->mode = FLY;
      break;

    case CWALK:
#ifdef DEBUG
      fprintf( stderr, "Proc %d: CWALK received\n", myPid );
#endif
      Viewer->getCamera()->position.setValue( cPrms ); 
      Viewer->getCamera()->orientation.setValue( &cPrms[3] ); 
      Viewer->mode = WALK;
      break;
    }
  Viewer->render();
#ifdef DEBUG
  fprintf( stderr, "=proc: %d, end of sigio_handler\n\n", myPid );
#endif
}

void
SoXtWebViewer::showErrorMsg( char *title, char *message )
{
  Arg args[8];
  int nArgs = 0;
  XmString string = XmStringCreateSimple( title );
  XtSetArg( args[nArgs], XmNdialogTitle, string ); nArgs++;
  XtSetArg( args[nArgs], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL ); nArgs++;
  XtSetArg( args[nArgs], XmNresizePolicy, XmRESIZE_ANY ); nArgs++;
  XtSetArg( args[nArgs], XmNnoResize, False ); nArgs++;
  XtSetArg( args[nArgs], XmNautoUnmanage, True ); nArgs++;
  XtSetArg( args[nArgs], XmNdefaultPosition, True ); nArgs++;
  XtSetArg( args[nArgs], XmNallowShellResize, False ); nArgs++;
  XmString msg = XmStringCreateLtoR( message, XmSTRING_DEFAULT_CHARSET );
  XtSetArg( args[nArgs], XmNmessageString, msg ); nArgs++;
     
  Widget errDialog = XmCreateErrorDialog( getWidget(), "ErrorDialog", args, nArgs );
  XmStringFree( string );
  XmStringFree( msg );
     
  Widget w = XmMessageBoxGetChild( errDialog, XmDIALOG_OK_BUTTON );
  XtUnmanageChild( w );
  w = XmMessageBoxGetChild( errDialog, XmDIALOG_HELP_BUTTON );
  XtUnmanageChild( w );
     
  XtAddCallback( errDialog, XmNcancelCallback, ( XtCallbackProc )errDialogCB, NULL );
  XtManageChild( errDialog );
  XSync( getDisplay(), False );
  XmUpdateDisplay( errDialog );
}
